package core.api.controller;

import core.api.models.auth.SimpleStatusResponse;
import core.api.models.timemanager.history.TaskgroupActivityInfo;
import core.api.models.timemanager.history.WorkingStatus;
import core.entities.timemanager.AbstractSchedule;
import core.entities.timemanager.Taskgroup;
import core.services.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.bind.annotation.*;

import java.time.*;
import java.time.format.DateTimeFormatter;
import java.util.*;

@CrossOrigin(origins = "*", maxAge = 3600)
@RestController
@RequestMapping("/api")
public class StatisticController {

    @Autowired private TaskScheduleService taskScheduleService;
    @Autowired private StatisticService statisticService;
    @Autowired private TaskgroupService taskgroupService;

    @GetMapping("/history/workingStatus")
    public ResponseEntity<?> getWorkingStatus() {
        boolean missedSchedules = !taskScheduleService.getAllMissedSchedulesOfUser(SecurityContextHolder.getContext().getAuthentication().getName()).isEmpty();
        List<AbstractSchedule> executedSchedules = taskScheduleService.getActiveSchedulesOfToday(SecurityContextHolder.getContext().getAuthentication().getName());
        int activeTime = 0;
        for(AbstractSchedule schedule : executedSchedules) {
            if(schedule.getStartTime().toLocalDate().isBefore(LocalDate.now())) {
                activeTime += (int) Duration.between(LocalDateTime.of(LocalDate.now(), LocalTime.MIDNIGHT), schedule.getStopTime()).toMinutes();
            } else {
                activeTime += (int) Duration.between(schedule.getStartTime(), schedule.getStopTime()).toMinutes();
            }
        }

        return ResponseEntity.ok(new WorkingStatus(missedSchedules, activeTime));
    }

    @GetMapping("/statistics/{startingDate}/{endingDate}")
    public ResponseEntity<?> getTaskgroupActivity(@PathVariable String startingDate, @PathVariable String endingDate){
        LocalDate starting = LocalDate.parse(startingDate, DateTimeFormatter.ofPattern("yyyy-MM-dd"));
        LocalDate ending = LocalDate.parse(endingDate, DateTimeFormatter.ofPattern("yyyy-MM-dd"));

        var taskgroupActivityInfos = statisticService.calcActivityByUser(null, SecurityContextHolder.getContext().getAuthentication().getName(), starting, ending);
        List<TaskgroupActivityInfo> outgoingResult = StatisticService.convertInternActivityInfo(taskgroupActivityInfos);
        return ResponseEntity.ok(outgoingResult);
    }

    @GetMapping("/statistics/{taskgroupID}/{startingDate}/{endingDate}")
    public ResponseEntity<?> getTaskgroupActivity(@PathVariable long taskgroupID, @PathVariable String startingDate, @PathVariable String endingDate){
        LocalDate starting = LocalDate.parse(startingDate, DateTimeFormatter.ofPattern("yyyy-MM-dd"));
        LocalDate ending = LocalDate.parse(endingDate, DateTimeFormatter.ofPattern("yyyy-MM-dd"));
        String username = SecurityContextHolder.getContext().getAuthentication().getName();
        var permissionResult = taskgroupService.getTaskgroupByIDAndUsername(taskgroupID, username);
        if(permissionResult.hasIssue()) {
            return permissionResult.mapToResponseEntity();
        }

        var taskgroupActivityInfos = statisticService.calcActivityByUser(permissionResult.getResult(), username, starting, ending);
        List<TaskgroupActivityInfo> outgoingResult = StatisticService.convertInternActivityInfo(taskgroupActivityInfos);
        return ResponseEntity.ok(outgoingResult);
    }

    @GetMapping("/history/schedules/{date}")
    public ResponseEntity<?> getPastSchedules(@PathVariable String date) {
        LocalDate dateArg = LocalDate.parse(date, DateTimeFormatter.ofPattern("yyyy-MM-dd"));
        List<AbstractSchedule> abstractSchedules = taskScheduleService.findSchedulesByDate(SecurityContextHolder.getContext().getAuthentication().getName(), dateArg);
        return ResponseEntity.ok(abstractSchedules.stream().map(AbstractSchedule::toScheduleInfo).toList());
    }
}
