import {Component, OnInit, ViewChild} from '@angular/core';
import {
  BasicScheduleEntityInfo, HistoryService,
  ScheduleInfo,
  ScheduleService,
  TaskOverviewInfo,
  TaskScheduleStopResponse
} from "../../api";
import {ActiveScheduleComponent, StopActiveScheduleEmitterInfo} from "./active-schedule/active-schedule.component";
import {StopActiveScheduleInfo} from "./active-schedule/StopActiveScheduleInfo";
import {TaskCreationEmitterInfo, TaskOverviewComponent} from "./task-overview/task-overview.component";
import {TaskgroupOverviewComponent, TaskOverviewData} from "./taskgroup-overview/taskgroup-overview.component";

@Component({
  selector: 'app-dashboard',
  templateUrl: './dashboard.component.html',
  styleUrls: ['./dashboard.component.css']
})
export class DashboardComponent implements OnInit{

  missedSchedules: boolean = false
  schedules: ScheduleInfo[] = []
  workedMinutesToday: number = 0

  tasks: TaskOverviewInfo[] = []
  selectedTaskgroupID: number | undefined

  @ViewChild('activeSchedule') activeScheduleComponent: ActiveScheduleComponent | undefined
  @ViewChild('taskgroupOverview') taskroupOverviewComponent: TaskgroupOverviewComponent | undefined
  constructor(private scheduleService: ScheduleService,
              private historyService: HistoryService) {
  }

  ngOnInit() {
    const date: Date = new Date();
    console.log(date)
    this.scheduleService.schedulesDateStartableGet(date.toISOString().split("T")[0],  true).subscribe({
      next: resp => {
        this.schedules = resp;
      }
    })

    this.historyService.historyWorkingStatusGet().subscribe({
      next: resp => {
        this.workedMinutesToday = resp.activeMinutes;
        this.missedSchedules = resp.missedSchedules;
      }
    })
  }

  startSchedule(schedule: ScheduleInfo) {
    this.scheduleService.schedulesScheduleIDActivatePost(schedule.scheduleID).subscribe({
      next: resp => {
        schedule.startTime = resp.startTime;
        if(this.activeScheduleComponent != undefined) {
          this.activeScheduleComponent.activateSchedule(schedule);
        }
      }
    })
  }


  stopedTask(stopActiveScheduleInfo: StopActiveScheduleEmitterInfo) {
    this.workedMinutesToday += stopActiveScheduleInfo.stopactiveScheduleResponse.workedMinutes;
    this.schedules = this.schedules.filter(schedule => schedule.scheduleID !== stopActiveScheduleInfo.stopactiveScheduleResponse.schedule.scheduleID);

    if(stopActiveScheduleInfo.finish) {
      this.taskroupOverviewComponent!.finishTask(stopActiveScheduleInfo.taskID);
    }
  }


    protected readonly TaskOverviewComponent = TaskOverviewComponent;

  onSelectTaskgroup(taskOverviewData: TaskOverviewData) {
    this.tasks = taskOverviewData.tasks;
    this.selectedTaskgroupID = taskOverviewData.taskgroupID;
  }

  onStartTaskNow(schedule: ScheduleInfo) {
    this.activeScheduleComponent?.activateSchedule(schedule);
    this.selectedTaskgroupID = undefined
  }

  onFinishTask(task: TaskOverviewInfo) {
    this.activeScheduleComponent?.finishTaskByOverview(task);

    this.schedules = this.schedules.filter(schedule => schedule.task.taskID !== task.taskID)
    this.taskroupOverviewComponent!.finishTask(task.taskID);
    this.tasks = [];
    this.selectedTaskgroupID = undefined;
  }

  registerForgotten(taskScheduleStopResponse: TaskScheduleStopResponse) {
    this.workedMinutesToday += taskScheduleStopResponse.workTime
  }

  onCreatedTask(taskCreationInfo: TaskCreationEmitterInfo) {
    this.taskroupOverviewComponent!.onCreateTask(taskCreationInfo);
  }
}
