import {Component, OnInit} from '@angular/core';
import {ScheduleService, TaskEntityInfo, TaskService, TaskShortInfo, TaskTaskgroupInfo} from "../../api";
import {NavigationLink} from "../navigation-link-list/navigation-link-list.component";
import {Router} from "@angular/router";
import {MatSnackBar} from "@angular/material/snack-bar";

@Component({
  selector: 'app-overdue-task-overview',
  templateUrl: './overdue-task-overview.component.html',
  styleUrls: ['./overdue-task-overview.component.css']
})
export class OverdueTaskOverviewComponent implements OnInit{

  overdueTasks: TaskTaskgroupInfo[] = []
  defaultNavigationLinkPath: NavigationLink[] = [
    {
      linkText: "Dashboard",
      routerLink: ['/']
    },
    {
      linkText: "Overdue Tasks",
      routerLink: ["/overdue"]
    }
  ]
  constructor(private taskService: TaskService,
              private scheduleService: ScheduleService,
              private router: Router,
              private snackbar: MatSnackBar) {

  }

  ngOnInit() {
    this.taskService.tasksAllScopeDetailedGet("OVERDUE", true).subscribe({
      next: resp => {
        this.overdueTasks = resp as TaskTaskgroupInfo[];
      }
    })
  }

  calcProgress(task: TaskTaskgroupInfo): number {
    console.log(task.workTime / task.eta * 100)
    return task.workTime / task.eta * 100
  }
  startNow(task: TaskTaskgroupInfo) {
    this.scheduleService.schedulesTaskIDNowPost(task.taskID).subscribe({
      next: resp => {
        this.router.navigateByUrl("/");
      },
      error: err => {
        if(err.status == 403) {
          this.snackbar.open("No permissions", "", {duration: 2000});
        } else if(err.status == 404) {
          this.snackbar.open("Not found", "", {duration: 2000});
        } else if(err.status == 409) {
          this.snackbar.open("Task already running", "", {duration: 2000});
        } else {
          this.snackbar.open("Unexpected error", "", {duration: 2000});
        }
      }
    })
  }

  finishTask(task: TaskTaskgroupInfo) {
    this.taskService.tasksTaskIDFinishPost(task.taskID).subscribe({
      next: resp => {
        this.overdueTasks = this.overdueTasks.filter(t => t.taskID !== task.taskID)
      },
      error: err => {
        if(err.status == 403) {
          this.snackbar.open("No permissions", "", {duration: 2000});
        } else if(err.status == 404) {
          this.snackbar.open("Not found", "", {duration: 2000});
        } else {
          this.snackbar.open("Unexpected error", "", {duration: 2000});
        }
      }
    })
  }

  deleteTask(deletedTask: TaskTaskgroupInfo) {
    this.taskService.tasksTaskIDDelete(deletedTask.taskID).subscribe({
      next: resp => {
        this.overdueTasks = this.overdueTasks.filter(task => task.taskID !== deletedTask.taskID);
      },
      error: err => {
        if(err.status == 403) {
          this.snackbar.open("No permissions", "", {duration: 2000});
        } else if(err.status == 404) {
          this.snackbar.open("Not found", "", {duration: 2000});
        } else {
          this.snackbar.open("Unexpected error", "", {duration: 2000});
        }
      }
    })
  }


}
